@@ -2,7 +2,7 @@ package App::perlbrew;
 use strict;
 use warnings;
 use 5.008;
-our $VERSION = "0.73";
+our $VERSION = "0.75";
 use Config;
 
 BEGIN {
@@ -19,6 +19,7 @@ BEGIN {
     @INC = @oldinc;
 }
 
+use File::Glob 'bsd_glob';
 use Getopt::Long ();
 
 sub min(@) {
@@ -128,20 +129,20 @@ sub files_are_the_same {
     my %commands = (
         curl => {
             test     => '--version >/dev/null 2>&1',
-            get      => '--insecure --silent --location --fail -o - {url}',
-            download => '--insecure --silent --location --fail -o {output} {url}',
+            get      => '--silent --location --fail -o - {url}',
+            download => '--silent --location --fail -o {output} {url}',
             order    => 1,
         },
         wget => {
             test     => '--version >/dev/null 2>&1',
-            get      => '--no-check-certificate --quiet -O - {url}',
-            download => '--no-check-certificate --quiet -O {output} {url}',
+            get      => '--quiet -O - {url}',
+            download => '--quiet -O {output} {url}',
             order    => 2,
         },
         fetch => {
             test     => '--version >/dev/null 2>&1',
-            get      => '--no-verify-peer -o - {url}',
-            download => '--no-verify-peer {url}',
+            get      => '-o - {url}',
+            download => '-o {output} {url}',
             order    => 3,
         }
     );
@@ -280,10 +281,11 @@ sub new {
         U => [],
         A => [],
         sitecustomize => '',
+        destdir => '',
         noman => '',
         variation => '',
         both => [],
-	append => '',
+        append => '',
     );
 
     $opt{$_} = '' for keys %flavor;
@@ -323,9 +325,9 @@ sub parse_cmdline {
     return Getopt::Long::GetOptions(
         $params,
 
-        'force|f!',
-        'notest|n!',
-        'quiet|q!',
+        'force|f',
+        'notest|n',
+        'quiet|q',
         'verbose|v',
         'as=s',
       	'append=s',
@@ -335,6 +337,7 @@ sub parse_cmdline {
         'switch',
         'all',
         'shell=s',
+        'no-patchperl',
 
         # options passed directly to Configure
         'D=s@',
@@ -344,6 +347,7 @@ sub parse_cmdline {
         'j=i',
         # options that affect Configure and customize post-build
         'sitecustomize=s',
+        'destdir=s',
         'noman',
 
         # flavors support
@@ -366,6 +370,16 @@ sub root {
     return $self->{root} || $PERLBREW_ROOT;
 }
 
+sub home {
+    my ($self, $new_home) = @_;
+
+    if (defined($new_home)) {
+        $self->{home} = $new_home;
+    }
+
+    return $self->{home} || $PERLBREW_HOME;
+}
+
 sub current_perl {
     my ($self, $v) = @_;
     $self->{current_perl} = $v if $v;
@@ -703,13 +717,15 @@ sub available_perls {
 sub perl_release {
     my ($self, $version) = @_;
 
+    my $mirror = $self->cpan_mirror();
+
     # try src/5.0 symlinks, either perl-5.X or perl5.X; favor .tar.bz2 over .tar.gz
     my $index = http_get("http://www.cpan.org/src/5.0/");
     if ($index) {
         for my $prefix ( "perl-", "perl" ){
             for my $suffix ( ".tar.bz2", ".tar.gz" ) {
                 my $dist_tarball = "$prefix$version$suffix";
-                my $dist_tarball_url = $self->cpan_mirror() . "/src/5.0/$dist_tarball";
+                my $dist_tarball_url = "$mirror/src/5.0/$dist_tarball";
                 return ( $dist_tarball, $dist_tarball_url )
                     if ( $index =~ /href\s*=\s*"\Q$dist_tarball\E"/ms );
             }
@@ -724,14 +740,11 @@ sub perl_release {
 
     if ($x) {
         my $dist_tarball = (split("/", $x))[-1];
-        my $dist_tarball_url = $self->cpan_mirror() . "/authors/id/$x";
+        my $dist_tarball_url = "$mirror/authors/id/$x";
         return ($dist_tarball, $dist_tarball_url);
     }
 
-    # try to find it on search.cpan.org
-    my $mirror = $self->config->{mirror};
-    my $header = $mirror ? { 'Cookie' => "cpan=$mirror->{url}" } : undef;
-    my $html = http_get("http://search.cpan.org/dist/perl-${version}", $header);
+    my $html = http_get("http://search.cpan.org/dist/perl-${version}", { 'Cookie' => "cpan=$mirror" });
 
     unless ($html) {
         die "ERROR: Failed to locate perl-${version} tarball.";
@@ -807,10 +820,10 @@ sub run_command_init {
     }
 
     my $root_dir = $self->path_with_tilde($self->root);
-    my $pb_home_dir = $self->path_with_tilde($PERLBREW_HOME);
+    my $pb_home_dir = $self->path_with_tilde($self->home);
 
     my $code = qq(    source $root_dir/etc/${shrc});
-    if ($PERLBREW_HOME ne joinpath($self->env('HOME'), ".perlbrew")) {
+    if ($self->home ne joinpath($self->env('HOME'), ".perlbrew")) {
         $code = "    export PERLBREW_HOME=$pb_home_dir\n" . $code;
     }
 
@@ -1211,7 +1224,7 @@ sub run_command_download {
 
 sub purify {
     my ($self, $envname) = @_;
-    my @paths = grep { index($_, $PERLBREW_HOME) < 0 && index($_, $self->root) < 0 } split /:/, $self->env($envname);
+    my @paths = grep { index($_, $self->home) < 0 && index($_, $self->root) < 0 } split /:/, $self->env($envname);
     return wantarray ? @paths : join(":", @paths);
 }
 
@@ -1292,6 +1305,7 @@ sub do_install_this {
     my @u_options = @{ $self->{U} };
     my @a_options = @{ $self->{A} };
     my $sitecustomize = $self->{sitecustomize};
+    my $destdir = $self->{destdir};
     $installation_name = $self->{as} if $self->{as};
     $installation_name .= "$variation$append";
 
@@ -1311,8 +1325,8 @@ sub do_install_this {
         $self->{$flavor} and push @d_options, $flavor{$flavor}{d_option}
     }
 
-    my $perlpath = $self->root . "/perls/$installation_name";
-    my $patchperl = $self->root . "/bin/patchperl";
+    my $perlpath = joinpath($self->root, "perls", $installation_name);
+    my $patchperl = joinpath($self->root, "bin", "patchperl");
 
     unless (-x $patchperl && -f _) {
         $patchperl = "patchperl";
@@ -1342,8 +1356,8 @@ INSTALL
     my @preconfigure_commands = (
         "cd $dist_extracted_dir",
         "rm -f config.sh Policy.sh",
-        $patchperl,
     );
+    push @preconfigure_commands, $patchperl unless $self->{"no-patchperl"};
 
     my $configure_flags = $self->env("PERLBREW_CONFIGURE_FLAGS") || '-de';
 
@@ -1375,7 +1389,9 @@ INSTALL
     local $ENV{TEST_JOBS}=$self->{j}
       if $test_target eq "test_harness" && ($self->{j}||1) > 1;
 
-    my @install_commands = $self->{notest} ? "make install" : ("make $test_target", "make install");
+    my @install_commands = ("make install" . ($destdir ? " DESTDIR=$destdir" : q||));
+    unshift @install_commands, "make $test_target" unless $self->{notest};
+    # Whats happening here? we optionally join with && based on $self->{force}, but then subsequently join with && anyway?
     @install_commands    = join " && ", @install_commands unless($self->{force});
 
     my $cmd = join " && ",
@@ -1395,16 +1411,23 @@ INSTALL
         $cmd = "($cmd) >> '$self->{log_file}' 2>&1 ";
     }
 
-    delete $ENV{$_} for qw(PERL5LIB PERL5OPT);
+    delete $ENV{$_} for qw(PERL5LIB PERL5OPT AWKPATH);
 
     if ($self->do_system($cmd)) {
         my $newperl = joinpath($self->root, "perls", $installation_name, "bin", "perl");
         unless (-e $newperl) {
             $self->run_command_symlink_executables($installation_name);
         }
+
+        eval { $self->append_log('##### Brew Finished #####') };
+
         if ( $sitecustomize ) {
             my $capture = $self->do_capture("$newperl -V:sitelib");
-            my ($sitelib) = $capture =~ /sitelib='(.*)';/;
+            my ($sitelib) = $capture =~ m/sitelib='([^']*)';/;
+            # This should probably all use File::Path
+            if ($destdir) {
+                $sitelib = $destdir . $sitelib
+            }
             mkpath($sitelib) unless -d $sitelib;
             my $target = "$sitelib/sitecustomize.pl";
             open my $dst, ">", $target
@@ -1425,6 +1448,7 @@ INSTALL
         print "$installation_name is successfully installed.\n";
     }
     else {
+        eval { $self->append_log('##### Brew Failed #####') };
         die $self->INSTALLATION_FAILURE_MESSAGE;
     }
     return;
@@ -1433,7 +1457,7 @@ INSTALL
 sub do_install_program_from_url {
     my ($self, $url, $program_name, $body_filter) = @_;
 
-    my $out = $self->root . "/bin/" . $program_name;
+    my $out = joinpath($self->root, "bin", $program_name);
 
     if (-f $out && !$self->{force}) {
         require ExtUtils::MakeMaker;
@@ -1513,10 +1537,10 @@ sub installed_perls {
     my @result;
     my $root = $self->root;
 
-    for (<$root/perls/*>) {
-        my ($name) = $_ =~ m/\/([^\/]+$)/;
-        my $executable = joinpath($_, 'bin', 'perl');
-        my $version_file = joinpath($_,'.version');
+    for my $installation_dir (<$root/perls/*>) {
+        my ($name) = $installation_dir =~ m/\/([^\/]+$)/;
+        my $executable = joinpath($installation_dir, 'bin', 'perl');
+        my $version_file = joinpath($installation_dir,'.version');
         my $orig_version;
         if ( -e $version_file ){
             open my $fh, '<', $version_file;
@@ -1538,7 +1562,8 @@ sub installed_perls {
             version     => $self->format_perl_version($orig_version),
             is_current  => ($self->current_perl eq $name) && !($self->current_lib),
             libs => [ $self->local_libs($name) ],
-            executable  => $executable
+            executable  => $executable,
+            dir => $installation_dir,
         };
     }
 
@@ -1548,24 +1573,21 @@ sub installed_perls {
 sub local_libs {
     my ($self, $perl_name) = @_;
 
-    my @libs = map { substr($_, length($PERLBREW_HOME) + 6) } <$PERLBREW_HOME/libs/*>;
-
-    if ($perl_name) {
-        @libs = grep { /^$perl_name\@/ } @libs;
-    }
-
     my $current = $self->current_perl . '@' . ($self->env("PERLBREW_LIB") || '');
-
-    @libs = map {
-        my ($p, $l) = split(/@/, $_);
-
+    my @libs = map {
+        my $name = substr($_, length($self->home) + 6);
+        my ($p, $l) = split(/@/, $name);
         +{
-            name       => $_,
-            is_current => $_ eq $current,
+            name       => $name,
+            is_current => $name eq $current,
             perl_name  => $p,
-            lib_name   => $l
+            lib_name   => $l,
+            dir        => $_,
         }
-    } @libs;
+    } bsd_glob(joinpath($self->home, "libs", "*"));
+    if ($perl_name) {
+        @libs = grep { $perl_name eq $_->{perl_name} } @libs;
+    }
     return @libs;
 }
 
@@ -1592,6 +1614,10 @@ sub perlbrew_env {
         unless ($perl_name) {
             die "\nERROR: The installation \"$name\" is unknown.\n\n";
         }
+
+        unless (!$lib_name || grep { $_->{lib_name} eq $lib_name } $self->local_libs($perl_name)) {
+            die "\nERROR: The lib name \"$lib_name\" is unknown.\n\n";
+        }
     }
 
     my %env = (
@@ -1602,10 +1628,11 @@ sub perlbrew_env {
     );
 
     require local::lib;
+    my $pb_home = $self->home;
     my $current_local_lib_root = $self->env("PERL_LOCAL_LIB_ROOT") || "";
     my $current_local_lib_context = local::lib->new;
-    my @perlbrew_local_lib_root =  uniq(grep { /\Q${PERLBREW_HOME}\E/ } split(/:/, $current_local_lib_root));
-    if ($current_local_lib_root =~ /^\Q$PERLBREW_HOME\E/) {
+    my @perlbrew_local_lib_root =  uniq(grep { /\Q${pb_home}\E/ } split(/:/, $current_local_lib_root));
+    if ($current_local_lib_root =~ /^\Q${pb_home}\E/) {
         $current_local_lib_context = $current_local_lib_context->activate($_) for @perlbrew_local_lib_root;
     }
 
@@ -1619,10 +1646,17 @@ sub perlbrew_env {
         if ($lib_name) {
             $current_local_lib_context = $current_local_lib_context->deactivate($_) for @perlbrew_local_lib_root;
 
-            my $base = "$PERLBREW_HOME/libs/${perl_name}\@${lib_name}";
+            my $base = joinpath($self->home, "libs", "${perl_name}\@${lib_name}");
 
             if (-d $base) {
                 $current_local_lib_context = $current_local_lib_context->activate($base);
+
+                if ( $self->env('PERLBREW_LIB_PREFIX') ) {
+                    unshift
+                        @{$current_local_lib_context->libs},
+                            $self->env('PERLBREW_LIB_PREFIX');
+                }
+
                 $env{PERLBREW_PATH}    = joinpath($base, "bin") . ":" . $env{PERLBREW_PATH};
                 $env{PERLBREW_MANPATH} = joinpath($base, "man") . ":" . $env{PERLBREW_MANPATH};
                 $env{PERLBREW_LIB}  = $lib_name;
@@ -1768,7 +1802,7 @@ sub switch_to {
     if ($self->env("PERLBREW_BASHRC_VERSION")) {
         local $ENV{PERLBREW_PERL} = $dist;
         my $HOME = $self->env('HOME');
-        my $pb_home = $self->env("PERLBREW_HOME") || $PERLBREW_HOME;
+        my $pb_home = $self->home;
 
         mkpath($pb_home);
         system("$0 env $dist > " . joinpath($pb_home, "init"));
@@ -1787,7 +1821,7 @@ sub run_command_off {
 
 sub run_command_switch_off {
     my $self = shift;
-    my $pb_home = $self->env("PERLBREW_HOME") || $PERLBREW_HOME;
+    my $pb_home = $self->home;
 
     mkpath($pb_home);
     system("env PERLBREW_PERL= $0 env > " . joinpath($pb_home, "init"));
@@ -1796,69 +1830,6 @@ sub run_command_switch_off {
     print "To immediately make it effective, run this line in this terminal:\n\n    exec @{[ $self->env('SHELL') ]}\n\n";
 }
 
-sub run_command_mirror {
-    my($self) = @_;
-    print "Fetching mirror list\n";
-    my $raw = http_get("http://search.cpan.org/mirror");
-
-    unless ($raw) {
-        die "\nERROR: Failed to retrieve the mirror list.\n\n";
-    }
-
-    my $found;
-    my @mirrors;
-    foreach my $line ( split m{\n}, $raw ) {
-        $found = 1 if $line =~ m{<select name="mirror">};
-        next if ! $found;
-        last if $line =~ m{</select>};
-        if ( $line =~ m{<option value="(.+?)">(.+?)</option>} ) {
-            my $url  = $1;
-            my $name = $2;
-            $name =~ s/&#(\d+);/chr $1/seg;
-            $url =~ s/&#(\d+);/chr $1/seg;
-            push @mirrors, { url => $url, name => $name };
-        }
-    }
-
-    require ExtUtils::MakeMaker;
-    my $select;
-    my $max = @mirrors;
-    my $id  = 0;
-    while ( @mirrors ) {
-        my @page = splice(@mirrors,0,20);
-        my $base = $id;
-        printf "[% 3d] %s\n", ++$id, $_->{name} for @page;
-        my $remaining = $max - $id;
-        my $ask = "Select a mirror by number or press enter to see the rest "
-                . "($remaining more) [q to quit, m for manual entry]";
-        my $val = ExtUtils::MakeMaker::prompt( $ask );
-        if ( ! length $val )  { next }
-        elsif ( $val eq 'q' ) { last }
-        elsif ( $val eq 'm' ) {
-            my $url  = ExtUtils::MakeMaker::prompt("Enter the URL of your CPAN mirror:");
-            my $name = ExtUtils::MakeMaker::prompt("Enter a Name: [default: My CPAN Mirror]") || "My CPAN Mirror";
-            $select = { name => $name, url => $url };
-            last;
-        }
-        elsif ( not $val =~ /\s*(\d+)\s*/ ) {
-            die "Invalid answer: must be 'q', 'm' or a number\n";
-        }
-        elsif (1 <= $val and $val <= $max) {
-            $select = $page[ $val - 1 - $base ];
-            last;
-        }
-        else {
-            die "Invalid ID: must be between 1 and $max\n";
-        }
-    }
-    die "You didn't select a mirror!\n" if ! $select;
-    print "Selected $select->{name} ($select->{url}) as the mirror\n";
-    my $conf = $self->config;
-    $conf->{mirror} = $select;
-    $self->_save_config;
-    return;
-}
-
 sub run_command_env {
     my($self, $name) = @_;
 
@@ -1943,7 +1914,7 @@ sub run_command_self_upgrade {
         die "Your perlbrew installation appears to be system-wide.  Please upgrade through your package manager.\n";
     }
 
-    http_get('http://get.perlbrew.pl', undef, sub {
+    http_get('https://raw.githubusercontent.com/gugod/App-perlbrew/master/perlbrew', undef, sub {
         my ( $body ) = @_;
 
         open my $fh, '>', $TMP_PERLBREW or die "Unable to write perlbrew: $!";
@@ -1975,16 +1946,21 @@ sub run_command_uninstall {
         exit(-1);
     }
 
-    my $dir = "@{[ $self->root ]}/perls/$target";
+    my @installed = $self->installed_perls(@_);
 
-    if (-l $dir) {
-        die "\nThe given name `$target` is an alias, not a real installation. Cannot perform uninstall.\nTo delete the alias, run:\n\n    perlbrew alias delete $target\n\n";
-    }
+    my ($to_delete) = grep { $_->{name} eq $target } @installed;
 
-    unless(-d $dir) {
-        die "'$target' is not installed\n";
+    die "'$target' is not installed\n" unless $to_delete;
+
+    my @dir_to_delete;
+    for (@{$to_delete->{libs}}) {
+        push @dir_to_delete, $_->{dir};
+    }
+    push @dir_to_delete, $to_delete->{dir};
+
+    for (@dir_to_delete) {
+        rmpath($_);
     }
-    exec 'rm', '-rf', $dir;
 }
 
 sub run_command_exec {
@@ -2004,7 +1980,7 @@ sub run_command_exec {
     if ($opts{with}) {
         my %installed = map { $_->{name} => $_ } map { ($_, @{$_->{libs}}) } $self->installed_perls;
 
-        my $d = ($opts{with} =~ / /) ? qr( +) : qr(,+);
+        my $d = ($opts{with} =~ m/ /) ? qr( +) : qr(,+);
         my @with = grep { $_ } map {
             my ($p,$l) = $self->resolve_installation_name($_);
             $p .= "\@$l" if $l;
@@ -2023,7 +1999,7 @@ sub run_command_exec {
 
     my $overall_success = 1;
     for my $i ( @exec_with ) {
-        next if -l $self->root . '/perls/' . $i->{name}; # Skip Aliases
+        next if -l joinpath($self->root, 'perls', $i->{name}); # Skip Aliases
         my %env = $self->perlbrew_env($i->{name});
         next if !$env{PERLBREW_PERL};
 
@@ -2176,7 +2152,7 @@ sub run_command_lib_create {
     }
 
     my $fullname = $perl_name . '@' . $lib_name;
-    my $dir = joinpath($PERLBREW_HOME,  "libs", $fullname);
+    my $dir = joinpath($self->home,  "libs", $fullname);
 
     if (-d $dir) {
         die "$fullname is already there.\n";
@@ -2203,7 +2179,7 @@ sub run_command_lib_delete {
 
     my $current  = $self->current_perl . '@' . $self->current_lib;
 
-    my $dir = joinpath($PERLBREW_HOME,  "libs", $fullname);
+    my $dir = joinpath($self->home,  "libs", $fullname);
 
     if (-d $dir) {
 
@@ -2225,7 +2201,7 @@ sub run_command_lib_delete {
 
 sub run_command_lib_list {
     my ($self) = @_;
-    my $dir = joinpath($PERLBREW_HOME,  "libs");
+    my $dir = joinpath($self->home,  "libs");
     return unless -d $dir;
 
     opendir my $dh, $dir or die "open $dir failed: $!";
@@ -2285,6 +2261,16 @@ sub run_command_upgrade_perl {
     print "Upgrading $current->{name} to $dist_version\n" unless $self->{quiet};
     local $self->{as}        = $current->{name};
     local $self->{dist_name} = $dist;
+
+    require Config ;
+    my @d_options = map { '-D' . $flavor{$_}->{d_option}} keys %flavor ;
+    my %sub_config = map { $_ => $Config{$_}} grep { /^config_arg\d/} keys %Config ;
+    for my $value ( values %sub_config ) {
+        my $value_wo_D = $value;
+        $value_wo_D =~ s/^-D//;
+        push @{$self->{D}} , $value_wo_D if grep {/$value/} @d_options;
+    }
+
     $self->do_install_release($dist, $dist_version);
 }
 
@@ -2360,49 +2346,6 @@ sub run_command_info {
     print $self->format_info_output(@_);
 }
 
-
-sub config {
-    my($self) = @_;
-    $self->_load_config if ! $CONFIG;
-    return $CONFIG;
-}
-
-sub config_file {
-    my ($self) = @_;
-    joinpath( $self->root, 'Config.pm' );
-}
-
-sub _save_config {
-    my($self) = @_;
-    require Data::Dumper;
-    open my $FH, '>', $self->config_file or die "Unable to open config (@{[ $self->config_file ]}): $!";
-    my $d = Data::Dumper->new([$CONFIG],['App::perlbrew::CONFIG']);
-    print $FH $d->Dump;
-    close $FH;
-}
-
-sub _load_config {
-    my($self) = @_;
-
-    if ( ! -e $self->config_file ) {
-        local $CONFIG = {} if ! $CONFIG;
-        $self->_save_config;
-    }
-
-    open my $FH, '<', $self->config_file or die "Unable to open config (@{[ $self->config_file ]}): $!\n";
-    my $raw = do { local $/; my $rv = <$FH>; $rv };
-    close $FH;
-
-    my $rv = eval $raw;
-    if ( $@ ) {
-        warn "Error loading conf: $@\n";
-        $CONFIG = {};
-        return;
-    }
-    $CONFIG = {} if ! $CONFIG;
-    return;
-}
-
 sub BASHRC_CONTENT() {
     return "export PERLBREW_BASHRC_VERSION=$VERSION\n" .
            (exists $ENV{PERLBREW_ROOT} ? "export PERLBREW_ROOT=$PERLBREW_ROOT\n" : "") . "\n" . <<'RC';
@@ -2478,12 +2421,8 @@ perlbrew () {
                 [ -n "$PERLBREW_LIB" ] && echo -n "@$PERLBREW_LIB"
                 echo
             else
-                __perlbrew_set_env "$2"
+                __perlbrew_set_env "$2" && { __perlbrew_set_path ; true ; }
                 exit_status="$?"
-                if [[ $exit_status -eq 0 ]]
-                then
-                    __perlbrew_set_path
-                fi
             fi
             ;;
 
@@ -2491,11 +2430,8 @@ perlbrew () {
               if [[ -z "$2" ]] ; then
                   command perlbrew switch
               else
-                  perlbrew use $2
+                  perlbrew use $2 && { __perlbrew_reinit $2 ; true ; }
                   exit_status=$?
-                  if [[ ${exit_status} -eq 0 ]]; then
-                      __perlbrew_reinit $2
-                  fi
               fi
               ;;
 
@@ -2583,16 +2519,17 @@ function __perlbrew_set_path
 
     set -l PATH_WITHOUT_PERLBREW (eval $perlbrew_command display-pristine-path | perl -pe'y/:/ /')
 
+    # silencing stderr in case there's a non-existent path in $PATH (see GH#446)
     if test -n "$PERLBREW_PATH"
         set -x PERLBREW_PATH (echo $PERLBREW_PATH | perl -pe 'y/:/ /' )
-        eval set -x PATH $PERLBREW_PATH $PATH_WITHOUT_PERLBREW
+        eval set -x PATH $PERLBREW_PATH $PATH_WITHOUT_PERLBREW 2> /dev/null
     else
-        eval set -x PATH $PERLBREW_ROOT/bin $PATH_WITHOUT_PERLBREW
+        eval set -x PATH $PERLBREW_ROOT/bin $PATH_WITHOUT_PERLBREW 2> /dev/null
     end
 end
 
 function __perlbrew_set_env
-    set -l code (eval $perlbrew_command env $argv | perl -pe 's/^(export|setenv)/set -xg/; s/^unset[env]*/set -eug/; s/$/;/; y/:/ /')
+    set -l code (eval $perlbrew_command env $argv | perl -pe 's/^(export|setenv)/set -xg/; s/=/ /; s/^unset[env]*/set -eug/; s/$/;/; y/:/ /')
 
     if test -z "$code"
         return 0;
@@ -2670,7 +2607,7 @@ function perlbrew
 end
 
 function __source_init
-    perl -pe's/^\(export|setenv\)/set -xg/; s/=/ /; s/$/;/;' "$PERLBREW_HOME/init" | . -
+    perl -pe's/^(export|setenv)/set -xg/; s/=/ /; s/$/;/;' "$PERLBREW_HOME/init" | source
 end
 
 if test -z "$PERLBREW_ROOT"
@@ -2858,6 +2795,15 @@ CSHRC
 
 }
 
+sub append_log {
+    my ($self, $message) = @_;
+    my $log_handler;
+    open($log_handler, '>>', $self->{log_file})
+        or die "Cannot open log file for appending: $!";
+    print $log_handler "$message\n";
+    close($log_handler);
+}
+
 sub INSTALLATION_FAILURE_MESSAGE {
     my ($self) = @_;
     return <<FAIL;
@@ -2865,7 +2811,7 @@ Installation process failed. To spot any issues, check
 
   $self->{log_file}
 
-If some perl tests failed and you still want install this distribution anyway,
+If some perl tests failed and you still want to install this distribution anyway,
 do:
 
   (cd $self->{dist_extracted_dir}; make install)
@@ -2892,19 +2838,16 @@ __END__
 
 =head1 NAME
 
-App::perlbrew - Manage perl installations in your $HOME
+L<App::perlbrew> - Manage perl installations in your C<$HOME>
 
-=head1 SYNOPSIS
+=head2 SYNOPSIS
 
     # Installation
-    curl -kL http://install.perlbrew.pl | bash
+    curl -L http://install.perlbrew.pl | bash
 
     # Initialize
     perlbrew init
 
-    # Pick a preferred CPAN mirror
-    perlbrew mirror
-
     # See what is available
     perlbrew available
 
@@ -2933,28 +2876,29 @@ App::perlbrew - Manage perl installations in your $HOME
     # Exec something with all perlbrew-ed perls
     perlbrew exec -- perl -E 'say $]'
 
-=head1 DESCRIPTION
+=head2 DESCRIPTION
 
-perlbrew is a program to automate the building and installation of perl in an
+L<perlbrew> is a program to automate the building and installation of perl in an
 easy way. It provides multiple isolated perl environments, and a mechanism
 for you to switch between them.
 
 Everything are installed unter C<~/perl5/perlbrew>. You then need to include a
 bashrc/cshrc provided by perlbrew to tweak the PATH for you. You then can
-benefit from not having to run 'sudo' commands to install
-cpan modules because those are installed inside your HOME too.
+benefit from not having to run C<sudo> commands to install
+cpan modules because those are installed inside your C<HOME> too.
 
 For the documentation of perlbrew usage see L<perlbrew> command
-on CPAN, or by running C<perlbrew help>. The following documentation
+on L<MetaCPAN|https://metacpan.org/>, or by running C<perlbrew help>,
+or by visiting L<perlbrew's official website|http://perlbrew.pl/>. The following documentation
 features the API of C<App::perlbrew> module, and may not be remotely
 close to what your want to read.
 
-=head1 INSTALLATION
+=head2 INSTALLATION
 
 It is the simplest to use the perlbrew installer, just paste this statement to
 your terminal:
 
-    curl -kL http://install.perlbrew.pl | bash
+    curl -L http://install.perlbrew.pl | bash
 
 Or this one, if you have C<fetch> (default on FreeBSD):
 
@@ -2968,17 +2912,17 @@ The installed perlbrew command is a standalone executable that can be run with
 system perl. The minimum system perl version requirement is 5.8.0, which should
 be good enough for most of the OSes these days.
 
-A fat-packed version of C<patchperl> is also installed to
+A fat-packed version of L<patchperl> is also installed to
 C<~/perl5/perlbrew/bin>, which is required to build old perls.
 
 The directory C<~/perl5/perlbrew> will contain all install perl executables,
 libraries, documentations, lib, site_libs. In the documentation, that directory
-is referred as "perlbrew root". If you need to set it to somewhere else because,
-say, your HOME has limited quota, you can do that by setting C<PERLBREW_ROOT>
+is referred as C<perlbrew root>. If you need to set it to somewhere else because,
+say, your C<HOME> has limited quota, you can do that by setting C<PERLBREW_ROOT>
 environment variable before running the installer:
 
     export PERLBREW_ROOT=/opt/perl5
-    curl -kL http://install.perlbrew.pl | bash
+    curl -L http://install.perlbrew.pl | bash
 
 As a result, different users on the same machine can all share the same perlbrew
 root directory (although only original user that made the installation would
@@ -3004,27 +2948,27 @@ C<App::perlbrew> because it will be installed under a system PATH like
 C</usr/bin>, which is not affected by perlbrew C<switch> or C<use> command.
 
 The C<self-upgrade> command will not upgrade the perlbrew installed by cpan
-command, but it is also easy to upgrade perlbrew by running `cpan App::perlbrew`
+command, but it is also easy to upgrade perlbrew by running C<cpan App::perlbrew>
 again.
 
-=head1 METHODS
+=head2 METHODS
 
 =over 4
 
 =item (Str) current_perl
 
 Return the "current perl" object attribute string, or, if absent, the value of
-PERLBREW_PERL environment variable.
+C<PERLBREW_PERL> environment variable.
 
 =item (Str) current_perl (Str)
 
-Set the "current_perl" object attribute to the given value.
+Set the C<current_perl> object attribute to the given value.
 
 =back
 
-=head1 PROJECT DEVELOPMENT
+=head2 PROJECT DEVELOPMENT
 
-perlbrew project uses github
+L<perlbrew project|http://perlbrew.pl/> uses github
 L<http://github.com/gugod/App-perlbrew/issues> and RT
 <https://rt.cpan.org/Dist/Display.html?Queue=App-perlbrew> for issue
 tracking. Issues sent to these two systems will eventually be reviewed
@@ -3039,13 +2983,13 @@ Kang-min Liu  C<< <gugod@gugod.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2010,2011,2012,2013,2014,2015 Kang-min Liu C<< <gugod@gugod.org> >>.
+Copyright (c) 2010-2016 Kang-min Liu C<< <gugod@gugod.org> >>.
 
-=head1 LICENCE
+=head3 LICENCE
 
 The MIT License
 
-=head1 DISCLAIMER OF WARRANTY
+=head2 DISCLAIMER OF WARRANTY
 
 BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
 FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
